/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.sonar.api.batch.fs.InputFile;
import org.sonar.api.batch.rule.ActiveRules;
import org.sonar.api.batch.rule.CheckFactory;
import org.sonar.api.batch.sensor.internal.SensorContextTester;
import org.sonar.api.batch.sensor.issue.internal.DefaultNoSonarFilter;
import org.sonar.api.measures.CoreMetrics;
import org.sonar.api.measures.FileLinesContext;
import org.sonar.api.measures.FileLinesContextFactory;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.fail;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class CxxFileLinesContextTest {

  private FileLinesContextForTesting fileLinesContext;

  @BeforeEach
  public void setUp() throws IOException {
    ActiveRules rules = mock(ActiveRules.class);
    var checkFactory = new CheckFactory(rules);
    FileLinesContextFactory fileLinesContextFactory = mock(FileLinesContextFactory.class);
    fileLinesContext = new FileLinesContextForTesting();
    when(fileLinesContextFactory.createFor(Mockito.any(InputFile.class))).thenReturn(fileLinesContext);

    File baseDir = TestUtils.loadResource("/com/keyware/sonar/cxx");
    var context = SensorContextTester.create(baseDir);
    var inputFile = TestUtils.buildInputFile(baseDir, "ncloc.cc");
    context.fileSystem().add(inputFile);

    var sensor = new CxxSquidSensor(fileLinesContextFactory, checkFactory, new DefaultNoSonarFilter(), null);
    sensor.execute(context);
  }

  @Test
  void TestLinesOfCode() throws UnsupportedEncodingException, IOException {
    Set<Integer> linesOfCode = Stream.of(
      8, 10, 14, 16, 17, 21, 22, 23, 26, 31, 34, 35, 42, 44, 45, 49, 51, 53, 55, 56,
      58, 59, 63, 65, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 79, 82, 84, 86, 87, 89,
      90, 95, 98, 99, 100, 102, 107, 108, 109, 110, 111, 113, 115, 118, 119, 124, 126)
      .collect(Collectors.toCollection(HashSet::new));

    assertThat(fileLinesContext.linesOfCode).containsExactlyInAnyOrderElementsOf(linesOfCode);
  }

  @Test
  void TestExecutableLinesOfCode() throws UnsupportedEncodingException, IOException {
    assertThat(fileLinesContext.executableLines).containsExactlyInAnyOrder(
      10, 26, 34, 35, 56, 59, 69, 70, 72, 73,
      75, 76, 79, 87, 90, 98, 102, 118, 119, 126);
  }

  private class FileLinesContextForTesting implements FileLinesContext {

    public final Set<Integer> executableLines = new HashSet<>();
    public final Set<Integer> linesOfCode = new HashSet<>();

    @Override
    public void setIntValue(String metricKey, int line, int value) {
      assertThat(value).isEqualTo(1);

      switch (metricKey) {
        case CoreMetrics.NCLOC_DATA_KEY:
          linesOfCode.add(line);
          break;
        case CoreMetrics.EXECUTABLE_LINES_DATA_KEY:
          executableLines.add(line);
          break;
        default:
          fail("Unsupported metric key " + metricKey);
      }
    }

    @Override
    public void setStringValue(String metricKey, int line, String value) {
      fail("unexpected method called: setStringValue()");
    }

    @Override
    public void save() {
    }
  }

}
